@@ -67,7 +67,7 @@ static void  gdl_dock_item_get_property  (GObject      *object,
                                           GValue       *value,
                                           GParamSpec   *pspec);
 
-static void  gdl_dock_item_destroy       (GtkObject *object);
+static void  gdl_dock_item_destroy       (GtkWidget    *object);
 
 static void  gdl_dock_item_add           (GtkContainer *container,
                                           GtkWidget    *widget);
@@ -88,8 +88,6 @@ static void  gdl_dock_item_unmap         (GtkWidget *widget);
 static void  gdl_dock_item_realize       (GtkWidget *widget);
 static void  gdl_dock_item_style_set     (GtkWidget *widget,
                                           GtkStyle  *previous_style);
-static gint  gdl_dock_item_expose        (GtkWidget *widget,
-                                          GdkEventExpose *event);
 
 static gint  gdl_dock_item_button_changed (GtkWidget *widget,
                                            GdkEventButton *event);
@@ -199,13 +197,11 @@ gdl_dock_item_class_init (GdlDockItemClass *klass)
     static gboolean style_initialized = FALSE;
     
     GObjectClass       *g_object_class;
-    GtkObjectClass     *gtk_object_class;
     GtkWidgetClass     *widget_class;
     GtkContainerClass  *container_class;
     GdlDockObjectClass *object_class;
     
     g_object_class = G_OBJECT_CLASS (klass);
-    gtk_object_class = GTK_OBJECT_CLASS (klass);
     widget_class = GTK_WIDGET_CLASS (klass);
     container_class = GTK_CONTAINER_CLASS (klass);
     object_class = GDL_DOCK_OBJECT_CLASS (klass);
@@ -214,7 +210,7 @@ gdl_dock_item_class_init (GdlDockItemClass *klass)
     g_object_class->set_property = gdl_dock_item_set_property;
     g_object_class->get_property = gdl_dock_item_get_property;
 
-    gtk_object_class->destroy = gdl_dock_item_destroy;
+    widget_class->destroy = gdl_dock_item_destroy;
 
     widget_class->realize = gdl_dock_item_realize;
     widget_class->map = gdl_dock_item_map;
@@ -222,7 +218,6 @@ gdl_dock_item_class_init (GdlDockItemClass *klass)
     widget_class->size_request = gdl_dock_item_size_request;
     widget_class->size_allocate = gdl_dock_item_size_allocate;
     widget_class->style_set = gdl_dock_item_style_set;
-    widget_class->expose_event = gdl_dock_item_expose;
     widget_class->button_press_event = gdl_dock_item_button_changed;
     widget_class->button_release_event = gdl_dock_item_button_changed;
     widget_class->motion_notify_event = gdl_dock_item_motion;
@@ -601,7 +596,7 @@ gdl_dock_item_get_property  (GObject      *g_object,
 }
 
 static void
-gdl_dock_item_destroy (GtkObject *object)
+gdl_dock_item_destroy (GtkWidget *object)
 {
     GdlDockItem *item = GDL_DOCK_ITEM (object);
 
@@ -628,7 +623,7 @@ gdl_dock_item_destroy (GtkObject *object)
         g_free (priv);
     }
 
-    GTK_OBJECT_CLASS (gdl_dock_item_parent_class)->destroy (object);
+    GTK_WIDGET_CLASS (gdl_dock_item_parent_class)->destroy (object);
 }
 
 static void 
@@ -918,13 +913,12 @@ gdl_dock_item_realize (GtkWidget *widget)
     attributes.window_type = GDK_WINDOW_CHILD;
     attributes.wclass = GDK_INPUT_OUTPUT;
     attributes.visual = gtk_widget_get_visual (widget);
-    attributes.colormap = gtk_widget_get_colormap (widget);
     attributes.event_mask = (gtk_widget_get_events (widget) |
                              GDK_EXPOSURE_MASK |
                              GDK_BUTTON1_MOTION_MASK |
                              GDK_BUTTON_PRESS_MASK |
                              GDK_BUTTON_RELEASE_MASK);
-    attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+    attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
     window = gdk_window_new (gtk_widget_get_parent_window (widget),
                              &attributes, attributes_mask);
     gtk_widget_set_window (widget, window);
@@ -933,7 +927,6 @@ gdl_dock_item_realize (GtkWidget *widget)
     gtk_widget_style_attach (widget);
     gtk_style_set_background (gtk_widget_get_style (widget), window,
                               gtk_widget_get_state (GTK_WIDGET (item)));
-    gdk_window_set_back_pixmap (window, NULL, TRUE);
 
     if (item->child)
         gtk_widget_set_parent_window (item->child, window);
@@ -958,46 +951,9 @@ gdl_dock_item_style_set (GtkWidget *widget,
         gtk_style_set_background (gtk_widget_get_style (widget),
                                   window,
                                   gtk_widget_get_state (widget));
-        if (gtk_widget_is_drawable (widget))
-            gdk_window_clear (window);
     }
 }
 
-static void
-gdl_dock_item_paint (GtkWidget      *widget,
-                     GdkEventExpose *event)
-{
-    GdlDockItem  *item;
-
-    item = GDL_DOCK_ITEM (widget);
-
-    gtk_paint_box (gtk_widget_get_style (widget),
-                   gtk_widget_get_window (widget),
-                   gtk_widget_get_state (widget),
-                   GTK_SHADOW_NONE,
-                   &event->area, widget,
-                   "dockitem",
-                   0, 0, -1, -1);
-}
-
-static gint
-gdl_dock_item_expose (GtkWidget      *widget,
-                      GdkEventExpose *event)
-{
-    g_return_val_if_fail (widget != NULL, FALSE);
-    g_return_val_if_fail (GDL_IS_DOCK_ITEM (widget), FALSE);
-    g_return_val_if_fail (event != NULL, FALSE);
-
-    if (gtk_widget_is_drawable (widget) &&
-        event->window == gtk_widget_get_window (widget)) 
-    {
-        gdl_dock_item_paint (widget, event);
-        GTK_WIDGET_CLASS (gdl_dock_item_parent_class)->expose_event (widget,event);
-    }
-  
-    return FALSE;
-}
-
 #define EVENT_IN_GRIP_EVENT_WINDOW(ev,gr) \
     ((gr) != NULL && (ev)->window == GDL_DOCK_ITEM_GRIP (gr)->title_window)
 
@@ -1139,7 +1095,7 @@ gdl_dock_item_key_press (GtkWidget   *widget,
     gboolean event_handled = FALSE;
     
     if (GDL_DOCK_ITEM_IN_DRAG (widget)) {
-        if (event->keyval == GDK_Escape) {
+        if (event->keyval == GDK_KEY_Escape) {
             gdl_dock_item_drag_end (GDL_DOCK_ITEM (widget), TRUE);
             event_handled = TRUE;
         }
