@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- 
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- 
  *
  * This file is part of the GNOME Devtools Libraries.
  *
@@ -28,12 +28,12 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include "gdl-tools.h"
 #include "gdl-dock.h"
 #include "gdl-dock-master.h"
 #include "gdl-dock-paned.h"
 #include "gdl-dock-notebook.h"
 #include "gdl-dock-placeholder.h"
+#include "gdl-preview-window.h"
 
 #include "libgdlmarshal.h"
 
@@ -44,7 +44,6 @@
 /* ----- Private prototypes ----- */
 
 static void  gdl_dock_class_init      (GdlDockClass *class);
-static void  gdl_dock_instance_init   (GdlDock *dock);
 
 static GObject *gdl_dock_constructor  (GType                  type,
                                        guint                  n_construct_properties,
@@ -63,7 +62,7 @@ static void  gdl_dock_notify_cb       (GObject      *object,
 
 static void  gdl_dock_set_title       (GdlDock      *dock);
 
-static void  gdl_dock_destroy         (GtkObject    *object);
+static void  gdl_dock_destroy         (GtkWidget    *object);
 
 static void  gdl_dock_size_request    (GtkWidget      *widget,
                                        GtkRequisition *requisition);
@@ -124,8 +123,7 @@ struct _GdlDockPrivate
     gint                width;
     gint                height;
 
-    /* auxiliary fields */
-    GdkGC              *xor_gc;
+    GtkWidget          *area_window;
 };
 
 enum {
@@ -150,19 +148,17 @@ static guint dock_signals [LAST_SIGNAL] = { 0 };
 
 /* ----- Private functions ----- */
 
-GDL_CLASS_BOILERPLATE (GdlDock, gdl_dock, GdlDockObject, GDL_TYPE_DOCK_OBJECT);
+G_DEFINE_TYPE (GdlDock, gdl_dock, GDL_TYPE_DOCK_OBJECT);
 
 static void
 gdl_dock_class_init (GdlDockClass *klass)
 {
     GObjectClass       *g_object_class;
-    GtkObjectClass     *gtk_object_class;
     GtkWidgetClass     *widget_class;
     GtkContainerClass  *container_class;
     GdlDockObjectClass *object_class;
     
     g_object_class = G_OBJECT_CLASS (klass);
-    gtk_object_class = GTK_OBJECT_CLASS (klass);
     widget_class = GTK_WIDGET_CLASS (klass);
     container_class = GTK_CONTAINER_CLASS (klass);
     object_class = GDL_DOCK_OBJECT_CLASS (klass);
@@ -220,7 +216,7 @@ gdl_dock_class_init (GdlDockClass *klass)
                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT |
                           GDL_DOCK_PARAM_EXPORT));
     
-    gtk_object_class->destroy = gdl_dock_destroy;
+    widget_class->destroy = gdl_dock_destroy;
 
     widget_class->size_request = gdl_dock_size_request;
     widget_class->size_allocate = gdl_dock_size_allocate;
@@ -261,7 +257,7 @@ gdl_dock_class_init (GdlDockClass *klass)
 }
 
 static void
-gdl_dock_instance_init (GdlDock *dock)
+gdl_dock_init (GdlDock *dock)
 {
     gtk_widget_set_has_window (GTK_WIDGET (dock), FALSE);
 
@@ -295,17 +291,14 @@ gdl_dock_constructor (GType                  type,
                       GObjectConstructParam *construct_param)
 {
     GObject *g_object;
-    
-    g_object = GDL_CALL_PARENT_WITH_DEFAULT (G_OBJECT_CLASS, 
-                                               constructor, 
-                                               (type,
-                                                n_construct_properties,
-                                                construct_param),
-                                               NULL);
+
+    g_object = G_OBJECT_CLASS (gdl_dock_parent_class)-> constructor (type,
+                                                                     n_construct_properties,
+                                                                     construct_param);
     if (g_object) {
         GdlDock *dock = GDL_DOCK (g_object);
         GdlDockMaster *master;
-        
+
         /* create a master for the dock if none was provided in the construction */
         master = GDL_DOCK_OBJECT_GET_MASTER (GDL_DOCK_OBJECT (dock));
         if (!master) {
@@ -317,11 +310,11 @@ gdl_dock_constructor (GType                  type,
 
         if (dock->_priv->floating) {
             GdlDockObject *controller;
-            
+
             /* create floating window for this dock */
             dock->_priv->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
             g_object_set_data (G_OBJECT (dock->_priv->window), "dock", dock);
-            
+
             /* set position and default size */
             gtk_window_set_position (GTK_WINDOW (dock->_priv->window),
                                      GTK_WIN_POS_MOUSE);
@@ -330,32 +323,32 @@ gdl_dock_constructor (GType                  type,
                                          dock->_priv->height);
             gtk_window_set_type_hint (GTK_WINDOW (dock->_priv->window),
                                       GDK_WINDOW_TYPE_HINT_NORMAL);
-            
+
             /* metacity ignores this */
             gtk_window_move (GTK_WINDOW (dock->_priv->window),
                              dock->_priv->float_x,
                              dock->_priv->float_y);
-            
+
             /* connect to the configure event so we can track down window geometry */
             g_signal_connect (dock->_priv->window, "configure_event",
                               (GCallback) gdl_dock_floating_configure_event_cb,
                               dock);
-            
+
             /* set the title and connect to the long_name notify queue
-               so we can reset the title when this prop changes */
+             so we can reset the title when this prop changes */
             gdl_dock_set_title (dock);
             g_signal_connect (dock, "notify::long-name",
                               (GCallback) gdl_dock_notify_cb, NULL);
-            
+
             gtk_container_add (GTK_CONTAINER (dock->_priv->window), GTK_WIDGET (dock));
-    
+
             g_signal_connect (dock->_priv->window, "delete_event",
                               G_CALLBACK (gdl_dock_floating_window_delete_event_cb), 
                               NULL);
         }
         GDL_DOCK_OBJECT_SET_FLAGS (dock, GDL_DOCK_ATTACHED);
     }
-    
+
     return g_object;
 }
 
@@ -497,8 +490,6 @@ gdl_dock_notify_cb (GObject    *object,
 
     g_object_get (object, "long-name", &long_name, NULL);
 
-    g_message ("Notify long_name: %s", long_name);
-
     if (long_name)
     {
         dock = GDL_DOCK (object);
@@ -509,7 +500,7 @@ gdl_dock_notify_cb (GObject    *object,
 }
 
 static void
-gdl_dock_destroy (GtkObject *object)
+gdl_dock_destroy (GtkWidget *object)
 {
     GdlDock *dock = GDL_DOCK (object);
 
@@ -522,17 +513,17 @@ gdl_dock_destroy (GtkObject *object)
             priv->floating = FALSE;
             priv->window = NULL;
         }
-        
-        /* destroy the xor gc */
-        if (priv->xor_gc) {
-            g_object_unref (priv->xor_gc);
-            priv->xor_gc = NULL;
-        }
 
+        if (priv->area_window)
+        {
+            gtk_widget_destroy (priv->area_window);
+            priv->area_window = NULL;
+        }
+        
         g_free (priv);
     }
     
-    GDL_CALL_PARENT (GTK_OBJECT_CLASS, destroy, (object));
+   GTK_WIDGET_CLASS (gdl_dock_parent_class)->destroy (object);
 }
 
 static void
@@ -561,7 +552,7 @@ gdl_dock_size_request (GtkWidget      *widget,
     requisition->width += 2 * border_width;
     requisition->height += 2 * border_width;
 
-    widget->requisition = *requisition;
+    //gtk_widget_size_request (widget, requisition);    
 }
 
 static void
@@ -602,7 +593,7 @@ gdl_dock_map (GtkWidget *widget)
 
     dock = GDL_DOCK (widget);
 
-    GDL_CALL_PARENT (GTK_WIDGET_CLASS, map, (widget));
+    GTK_WIDGET_CLASS (gdl_dock_parent_class)->map (widget);
 
     if (dock->root) {
         child = GTK_WIDGET (dock->root);
@@ -622,7 +613,7 @@ gdl_dock_unmap (GtkWidget *widget)
 
     dock = GDL_DOCK (widget);
 
-    GDL_CALL_PARENT (GTK_WIDGET_CLASS, unmap, (widget));
+    GTK_WIDGET_CLASS (gdl_dock_parent_class)->unmap (widget);
 
     if (dock->root) {
         child = GTK_WIDGET (dock->root);
@@ -652,7 +643,7 @@ gdl_dock_show (GtkWidget *widget)
     g_return_if_fail (widget != NULL);
     g_return_if_fail (GDL_IS_DOCK (widget));
     
-    GDL_CALL_PARENT (GTK_WIDGET_CLASS, show, (widget));
+    GTK_WIDGET_CLASS (gdl_dock_parent_class)->show (widget);
     
     dock = GDL_DOCK (widget);
     if (dock->_priv->floating && dock->_priv->window)
@@ -673,7 +664,7 @@ gdl_dock_hide (GtkWidget *widget)
     g_return_if_fail (widget != NULL);
     g_return_if_fail (GDL_IS_DOCK (widget));
     
-    GDL_CALL_PARENT (GTK_WIDGET_CLASS, hide, (widget));
+    GTK_WIDGET_CLASS (gdl_dock_parent_class)->hide (widget);
     
     dock = GDL_DOCK (widget);
     if (dock->_priv->floating && dock->_priv->window)
@@ -793,7 +784,7 @@ gdl_dock_dock_request (GdlDockObject  *object,
     GdlDock            *dock;
     guint               bw;
     gint                rel_x, rel_y;
-    GtkAllocation      *alloc;
+    GtkAllocation       alloc;
     gboolean            may_dock = FALSE;
     GdlDockRequest      my_request;
 
@@ -804,28 +795,28 @@ gdl_dock_dock_request (GdlDockObject  *object,
     dock = GDL_DOCK (object);
     
     /* Get dock size. */
-    gtk_widget_get_allocation (GTK_WIDGET (dock), alloc);
+    gtk_widget_get_allocation (GTK_WIDGET (dock), &alloc);
     bw = gtk_container_get_border_width (GTK_CONTAINER (dock));
 
     /* Get coordinates relative to our allocation area. */
-    rel_x = x - alloc->x;
-    rel_y = y - alloc->y;
+    rel_x = x - alloc.x;
+    rel_y = y - alloc.y;
 
     if (request)
         my_request = *request;
         
     /* Check if coordinates are in GdlDock widget. */
-    if (rel_x > 0 && rel_x < alloc->width &&
-        rel_y > 0 && rel_y < alloc->height) {
+    if (rel_x > 0 && rel_x < alloc.width &&
+        rel_y > 0 && rel_y < alloc.height) {
 
         /* It's inside our area. */
         may_dock = TRUE;
 
 	/* Set docking indicator rectangle to the GdlDock size. */
-        my_request.rect.x = alloc->x + bw;
-        my_request.rect.y = alloc->y + bw;
-        my_request.rect.width = alloc->width - 2*bw;
-        my_request.rect.height = alloc->height - 2*bw;
+        my_request.rect.x = alloc.x + bw;
+        my_request.rect.y = alloc.y + bw;
+        my_request.rect.width = alloc.width - 2*bw;
+        my_request.rect.height = alloc.height - 2*bw;
 
 	/* If GdlDock has no root item yet, set the dock itself as 
 	   possible target. */
@@ -839,14 +830,14 @@ gdl_dock_dock_request (GdlDockObject  *object,
             if (rel_x < bw) {
                 my_request.position = GDL_DOCK_LEFT;
                 my_request.rect.width *= SPLIT_RATIO;
-            } else if (rel_x > alloc->width - bw) {
+            } else if (rel_x > alloc.width - bw) {
                 my_request.position = GDL_DOCK_RIGHT;
                 my_request.rect.x += my_request.rect.width * (1 - SPLIT_RATIO);
                 my_request.rect.width *= SPLIT_RATIO;
             } else if (rel_y < bw) {
                 my_request.position = GDL_DOCK_TOP;
                 my_request.rect.height *= SPLIT_RATIO;
-            } else if (rel_y > alloc->height - bw) {
+            } else if (rel_y > alloc.height - bw) {
                 my_request.position = GDL_DOCK_BOTTOM;
                 my_request.rect.y += my_request.rect.height * (1 - SPLIT_RATIO);
                 my_request.rect.height *= SPLIT_RATIO;
@@ -1328,45 +1319,25 @@ gdl_dock_object_get_toplevel (GdlDockObject *object)
 
 void
 gdl_dock_xor_rect (GdlDock      *dock,
-                   GdkRectangle *rect)
+                   cairo_rectangle_int_t *rect)
 {
-    GtkWidget *widget;
-    GdkWindow *window;
-    gint8      dash_list [2];
-
-    widget = GTK_WIDGET (dock);
-
-    if (!dock->_priv->xor_gc) {
-        if (gtk_widget_get_realized (widget)) {
-            GdkGCValues values;
-
-            values.function = GDK_INVERT;
-            values.subwindow_mode = GDK_INCLUDE_INFERIORS;
-            dock->_priv->xor_gc = gdk_gc_new_with_values 
-                (gtk_widget_get_window (widget), &values, GDK_GC_FUNCTION | GDK_GC_SUBWINDOW);
-        } else 
-            return;
-    };
-
-    gdk_gc_set_line_attributes (dock->_priv->xor_gc, 1,
-                                GDK_LINE_ON_OFF_DASH,
-                                GDK_CAP_NOT_LAST,
-                                GDK_JOIN_BEVEL);
-
-    window = gtk_widget_get_window (widget);
-
-    dash_list [0] = 1;
-    dash_list [1] = 1;
+    gint x, y;
+    GdkWindow* window = gtk_widget_get_window (GTK_WIDGET (dock));
+    gdk_window_get_origin (window, &x, &y);
     
-    gdk_gc_set_dashes (dock->_priv->xor_gc, 1, dash_list, 2);
+    if (!dock->_priv->area_window) {
+        dock->_priv->area_window = gdl_preview_window_new ();
+    }
 
-    gdk_draw_rectangle (window, dock->_priv->xor_gc, FALSE,
-                        rect->x, rect->y,
-                        rect->width, rect->height);
+    rect->x += x;
+    rect->y += y;
+    
+    gdl_preview_window_update (GDL_PREVIEW_WINDOW (dock->_priv->area_window), rect);
+}
 
-    gdk_gc_set_dashes (dock->_priv->xor_gc, 0, dash_list, 2);
-
-    gdk_draw_rectangle (window, dock->_priv->xor_gc, FALSE,
-                        rect->x + 1, rect->y + 1,
-                        rect->width - 2, rect->height - 2);
+void
+gdl_dock_xor_rect_hide (GdlDock      *dock)
+{
+    if (dock->_priv->area_window)
+        gtk_widget_hide (dock->_priv->area_window);
 }
